
/*!
 *************************************************************************************
 * \file adaptive_loop_filter.h
 *
 * \brief
 *    adaptive loop filter routines
 *
 * \author
 *    Takeshi Chujoh         <takeshi.chujoh@toshiba.co.jp>             \n
 *    Akiyuki Tanizawa       <akiyuki.tanizawa@toshiba.co.jp>           \n
 *    Goki Yasuda            <goki1.yasuda@toshiba.co.jp>               \n
 *    Naofumi Wada           <naofumi.wada@toshiba.co.jp>               \n
 *    Takashi Watanabe       <takashi39.watanabe@toshiba.co.jp>         \n
 *************************************************************************************
 */

#ifndef _ADAPTIVE_LOOP_FILTER_H_
#define _ADAPTIVE_LOOP_FILTER_H_

#include "global.h"

#ifdef ADAPTIVE_LOOP_FILTER

//------- defines
#define MIN_NUM_TAP    5
#define MAX_NUM_TAP    9
#define MAX_NUM_TAP_C  5
#define NUM_BIT_SHIFT  8
#define MAX_NUM_COEF   42
#define NUM_COEF_7x7   26
#define NUM_COEF_5x5   14
#define MAX_NUM_COEF_C 14
#define NUM_OF_REDESIGN 3
#define MAX_ALF_PARAM_SIZE 50000
//------- end


//------- prototype
alf_parameter_set_rbsp_t *AllocALFPS(int *memory_size);
void   FreeALFPS (alf_parameter_set_rbsp_t *alfps);
int    InitALFGlobalBuffers(void);
void   FreeALFGlobalBurrers(void);
void   SetALFParameters(alf_parameter_set_rbsp_t *alfps, alf_parameter_set_rbsp_t *p);
void   MakeExtendFrame( imgpel** ImgDec, imgpel** ImgExt, int width, int height, int offset );
void   SetRestFrame(imgpel** ImgRest, imgpel** ImgDec, int width, int height);
int    GetALFBaseType(int slice_type, int ref);
void   InitFilterBase();
void   SetALFDiffCoef(alf_parameter_set_rbsp_t *alfps);
void   SetALFBaseCoef(alf_parameter_set_rbsp_t *alfps);
void   SetALFDiffCoefChroma(alf_parameter_set_rbsp_t *alfps);
void   SetALFBaseCoefChroma(alf_parameter_set_rbsp_t *alfps);

void   CalcALFLambda(int qp);
int64  FindDistortionFrame(imgpel** ImgOrg, imgpel** ImgCmp);
void   FindDistortionFrameChroma(imgpel*** ImgUVOrg, imgpel*** ImgUVCmp, int64 *dist_u, int64 *dist_v);
int64  FindDistortionBlock(int blk_addr, imgpel** ImgOrg, imgpel** ImgCmp, int blk_size, int width_in_alf_blks);
void   CalcRDCost(imgpel** ImgOrg, imgpel** ImgCmp, alf_parameter_set_rbsp_t *alfps, int64* rate, int64* dist, double* cost);
void   CalcRDCostChroma(imgpel*** ImgUVOrg, imgpel*** ImgUVCmp, alf_parameter_set_rbsp_t *alfps, int64* rate, int64* dist, double* cost);
void   CalcRDCostBlock(imgpel** ImgOrg, imgpel** ImgCmp, imgpel** ImgDec, alf_parameter_set_rbsp_t *alfps, int64* rate, int64* dist, double* cost);
void   CalcCorrelationFunc(imgpel** ImgOrg, imgpel** ImgDec, double** corr, int tap, int width, int height);
void   CalcCorrelationFuncBlock(int blk_addr, imgpel** ImgOrg, imgpel** ImgDec, unsigned int** corr, int tap);
void   CalcCorrelationFuncBlockDiff(int blk_addr, imgpel** ImgOrg, imgpel** ImgDec, unsigned int** corr, int tap);
void   CopyBlockCorrelation(unsigned int **source, unsigned int **dest, int tap_idx);
void   CalcCorrelationFuncVariableTap(imgpel** ImgOrg, imgpel** ImgDec);
int    Gauss(double **a, int N);
void   FilterCoefQuickSort( double *coef_data, int *coef_num, int upper, int lower );
void   QuantFilterCoef(double* h, int* qh, int tap, int bitdepth);
void   ClearFilterCoefInt(int* qh, int N);

//------- filtering for luma
void   AdaptiveLoopFilterProcess(Picture *pic);
void   FilterTapDecision(imgpel** ImgOrg, imgpel** ImgDec, imgpel** ImgRest);
void   FilteringFrame(imgpel** ImgDec, imgpel** ImgRest);
void   FilteringProcess(imgpel** ImgDec, imgpel** ImgRest, int *qh, int tap);

//------- filtering for chroma
void   AdaptiveLoopFilterProcessChroma();
void   FilteringFrameChroma(imgpel ***ImgUVOrg, imgpel ***ImgUVDec, imgpel ***ImgUVRest);
void   FilteringProcessChroma(imgpel ***ImgUVDec, imgpel ***ImgUVRest, int *qh, int tap, int chroma_id);

//------- filtering based block
void   BlockAdaptiveFilterControl(imgpel** ImgOrg, imgpel** ImgDec, imgpel** ImgRest);
void   ReDesignFilterDecision(imgpel** ImgOrg, imgpel** ImgDec);
void   ReDesignFilterCoeff();
void   SetBlockSizeParameter(alf_parameter_set_rbsp_t *alfps, int size_idx);
int    SetALFBlockFlag(alf_parameter_set_rbsp_t *alfps, imgpel** ImgOrg, imgpel** ImgDec, imgpel** ImgRest);
void   CopyDecToRestBlock(int blk_addr, imgpel** ImgDec, imgpel** ImgRest, int blk_size, int width_in_alf_blks);
void   GetBlockPosForBALF(int blk_addr, int *x, int *y, int width_in_blks);
void   GetSamplePosForBALF(int blk_addr, int *x, int *y, int blk_size, int width_in_blks);
int    get_mem_blk_corr(unsigned int *****blk_corr, int height, int width, int rows, int columns);
void   free_mem_blk_corr(unsigned int ****blk_corr, int height, int width );
int    ALFBlockIsAvailable(int blk_addr);
void   CheckAvailabilityOfNeighborsAlf();
void   GetNeighbourAlfBlock(int curr_alfb_nr, int xN, int yN, PixelPos *pix);

//------- send ALF parameters
int    send_ALFParameterSet(alf_parameter_set_rbsp_t *alfps, Bitstream *bitstream);
int    send_ALFLumaParam(alf_parameter_set_rbsp_t *alfps, Bitstream *bitstream);
int    send_ALFChromaParam(alf_parameter_set_rbsp_t *alfps, Bitstream *bitstream);
int    send_ALFBlockControlParam(alf_parameter_set_rbsp_t *alfps, Bitstream *bitstream);
int    send_BlockFlagRunLength(alf_parameter_set_rbsp_t *alfps, Bitstream *bitstream);
int    send_BlockFlagCABAC(alf_parameter_set_rbsp_t *alfps, Bitstream *bitstream);
int    TerminateSliceForALF(Picture* pic);


#endif // ADAPTIVE_LOOP_FILTER

#endif // _ADAPTIVE_LOOP_FILTER_H_
